#!/usr/bin/env python3

## Loops over images in combined analysis directory
## Looks for cropped version of image and duplicates combined analysis dataset using cropped images
## Checks list of uncropped images for 'blanks' or filenaming errors and corrects
## Adds 'blank' images into test datasets

## Note: user needs to set file paths

### Created by Danielle Norman 30th November 2021

############### Imports ################
import json
import os
import subprocess
import pandas as pd
import numpy as np
import shutil
import json
from PIL import Image

##### File paths ######

dir_path = '/a/b/combined_analysis/' # path to combined_analysis datasets directory
data_path = '/c/d/e/' # path to dirctory in which to create MD datasets
cropped_path = '/f/g/h/' # path to cropped images
whole_path = '/i/j/k/' # path to whole images
outpath = '/x/y/z/' # path to store csvs created

## Generate list of all combined datasets
set_list=next(os.walk(dir_path))[1]

## Generate list of cropped images
listOfFilesMD = list()
for (dirpath, dirnames, filenames) in os.walk(cropped_path):
    listOfFilesMD += [os.path.join(file) for file in filenames]

# len(listOfFilesMD)

## loop over combined analysis directories
files_moved=list()
file_errors=list()
for s in set_list:
    listOfFiles = list()
    for (dirpath, dirnames, filenames) in os.walk(dir_path+s+'/'):
        listOfFiles += [os.path.join(file) for file in filenames]
    # check all cropped
    list_cropped = list(set(listOfFiles).intersection(set(listOfFilesMD)))
    if len(list_cropped) >0: # copy cropped files to new MD dataset directory
        for d in next(os.walk(dir_path+s))[1]:
            for spc in next(os.walk(dir_path+s+'/'+d))[1]:
                spc_files= next(os.walk(dir_path+s+'/'+d+'/'+spc))[2]
                # Create destination directories
                new_dir = data_path + s + '/' + d + '/' + spc
                subprocess.check_call("mkdir -p " + new_dir, shell=True)
                for file in spc_files:
                    try:
                        shutil.copyfile(cropped_path+file,new_dir + '/' + file) ##########
                        files_moved.append(s + '/' + d + '/' + spc + '/' + file)
                    except:
                        file_errors.append(s + '/' + d + '/' + spc + '/' + file)
#len(files_moved)
#len(file_errors)

############################# Check for missing files #############

## Generate list of images in combined analysis datasets
combi_files=list()
for (dirpath, dirnames, filenames) in os.walk(dir_path):
    combi_files += [os.path.join(file) for file in filenames]
# len(amends_files)
## remove duplicates
combi_files_unq = list(dict.fromkeys(combi_files))
len(combi_files_unq)

## Generate list of images in MD datasets
MD_files = list()
for (dirpath, dirnames, filenames) in os.walk(data_path):
    MD_files += [os.path.join(file) for file in filenames]
## remove duplictes
MD_files_unq = list(dict.fromkeys(MD_files))
len(MD_files_unq)

## Compare lists
missing_files = list(set(combi_files_unq).difference(set(MD_files_unq))) # should be equal to blanks list
len(missing_files)

##### Check list of missing files against list of 'blank' images #######

## Read in json and extract 'blank' images
f = open('imgs_cropped.json')
data = json.load(f) # dictionary
# Iterate through the json - extract detections - most confident per image and read in bbox coordinates
det_vehicle=list()
det_blank=list()
file_errors=list()
counter=0
for im in data['images']:
    file = im['file'].rsplit('/',1)[1]
    detections = im['detections']
    if detections == []:
        det_blank.append(file)
    counter +=1
## Close file
f.close()

len(det_blank)
##

queries = list(set(missing_files).difference(set(det_blank)))
len(queries)

## If there are queries, i.e. cropped files not moved for some reason, check they have been cropped
## Some filenaming mismatches - create list containing matches to combi dataset
queries.sort()
queries_fixed = list()
for f in queries:
    [strt,end] = f.split('_',1)
    if strt=='OG2-W-24' or strt=='OG1-N-8' or strt=='OG1-E-40' or strt=='OG1-W-38':
        f_new = strt + " " + end 
    elif strt=='D1-1-4':
        f_new = end
    elif strt=='IMG':
        f_new = 'OP3-N-16_' + f
    queries_fixed.append(f_new)

## Move files
files_moved_queried=list()
file_errors_queried=list()
for s in set_list:
    listOfFiles = list()
    for (dirpath, dirnames, filenames) in os.walk(dir_path+s+'/'):
        listOfFiles += [os.path.join(file) for file in filenames]
    # check against list of files to move
    list_match_queries = list(set(listOfFiles).intersection(set(queries)))
    if len(list_match_queries) >0: # move files from cropped directory
        for d in next(os.walk(dir_path+s))[1]:
            for spc in next(os.walk(dir_path+s+'/'+d))[1]:
                spc_files= next(os.walk(dir_path+s+'/'+d+'/'+spc))[2]
                # Create destination directories
                list_match_spc = list(set(spc_files).intersection(set(queries)))
                new_dir = data_path + s + '/' + d + '/' + spc
                for file in list_match_spc:
                    ind = queries.index(file)
                    try:
                        shutil.copyfile(cropped_path+queries_fixed[ind],new_dir + '/' + queries[ind]) ##########
                        files_moved_queried.append(s + '/' + d + '/' + spc + '/' + file)
                    except:
                        file_errors_queried.append(s + '/' + d + '/' + spc + '/' + file)

len(files_moved_queried)
len(file_errors_queried)

## Check whether remainings are blank - yes they are
df = pd.DataFrame(file_errors_queried,columns = ['fullpath'])
df['path'] = df.fullpath.str.rsplit('/',1,expand=True)[0]
df['file'] = df.fullpath.str.rsplit('/',1,expand=True)[1]
remaining_queries = list(df.file.unique())
remaining_queries_fixed=list()
for q in remaining_queries:
    remaining_queries_fixed.append(queries_fixed[queries.index(q)])
queries_blank = list(set(det_blank).intersection(set(remaining_queries_fixed)))

################################################################################
##### Move blanks to test directories #####

blank_files_moved=list()
test_files_error=list()
for s in set_list[0:5]:
    for spc in next(os.walk(dir_path+s+'/test/'))[1]:
        spc_files= next(os.walk(dir_path+s+'/test/'+spc))[2]
        # Create destination directories
        set_blanks = list(set(spc_files).intersection(set(det_blank)))
        new_dir = data_path + s + '/test/' + spc
        for file in set_blanks: # move resized blank images
            try:
                img = Image.open(whole_path + file)
                ## resize image
                img_res = img.resize((256,256))
                img_res.save(new_dir + '/' + file)
                blank_files_moved.append(new_dir + '/' + file)
            except:
                test_files_error.append(new_dir + '/' + file)

## Check sets are complete
complete_testsets = list()
incomplete_testsets = list()
test_files_moved=list()
test_files_error=list()
for s in set_list[0:5]:
    setlist_combi = list()
    setlist_MD = list()
    for (dirpath, dirnames, filenames) in os.walk(dir_path+s+'/test/'):
        setlist_combi += [os.path.join(file) for file in filenames]
    for (dirpath, dirnames, filenames) in os.walk(data_path+s+'/test/'):
        setlist_MD += [os.path.join(file) for file in filenames]
    if len(setlist_combi) == len(setlist_MD):
        complete_testsets.append(s)
    # else: # check against queries list
    #     test_queries = list(set(test_files_error).intersection(queries))
    #     if len(test_queries)==len(test_files_error):
    #         complete_testsets.append(s)
    else:
        incomplete_testsets.append(s)
        # incomplete_testsets.append(s)

len(complete_testsets) # 5



