#!/bin/bash
#Script: extract_metadata.sh
#Author: Danielle Norman, as based on extract_metadata.sh by Philipp Bischoff

#Description:

	# Camera trap metadata extraction script
	
	# INPUT: images in parent directory specified in variable images_dir
	# PROCESSES: Hidden image files and calibration and duplicate directories removed; spaces in filenames replaced with underscores
	# OUTPUT: image_info.csv: contains filepath, tags, event no. and date/time of creation 

	# Note: this script makes use of the 'exiftool' software
	# Variables to be specified: images_dir and out_file

#Date: 10/07/2019

#------------------------------------------------------------------------------
## Set images directory path and output csv file path
#------------------------------------------------------------------------------
images_dir=""
out_file="../Data/image_info_test.csv"

#------------------------------------------------------------------------------
## Remove hidden image files
#------------------------------------------------------------------------------
# INTRO MESSAGE
echo "
	====================================================
	Removing hidden image files
	====================================================
	"
find $images_dir -iname .*.jpg -type f -delete


#------------------------------------------------------------------------------
## Remove calibration directories
#------------------------------------------------------------------------------
# INTRO MESSAGE
echo "
	====================================================
	Removing calibration directories
	====================================================
	"
find $images_dir -type d -iname '*CALIB' -exec rm -r {} +

#------------------------------------------------------------------------------
## Remove duplicate directories
#------------------------------------------------------------------------------
# INTRO MESSAGE
echo "
	====================================================
	Removing duplicate directories
	====================================================
	"
find $images_dir -type d -iname '*Duplicate' -exec rm -r {} +

#------------------------------------------------------------------------------
## Replace spaces in filenames with underscores and add camera as prefix to ensure no duplicate filenames
#------------------------------------------------------------------------------
# INTRO MESSAGE
echo "
	====================================================
	Cleaning up filenames, by removing spaces
	====================================================
	"

find $images_dir -depth -name "* *" -execdir rename 's/ /_/g' "{}" \;
find $images_dir -iname '*.jpg' -exec sh -c 'mv $0 $(dirname $0)/$(basename $(dirname $0))_$(basename $0)' {} \; 


#------------------------------------------------------------------------------
## Making csv with relevant information
#------------------------------------------------------------------------------
echo "
	====================================================
	Creating metadata csv file. This will take a while.
	====================================================
	"

exiftool -csv -r -Subject -Sequence -EventNumber -dateTimeOriginal $images_dir > $out_file
